import java.io.*;
import java.util.*;

public class GenRandom {

	public final static Random rng = new Random("alternatingcycle".hashCode());
	public final static int MAX = 200_000;

	public final static int ARGS_COUNT = 3;

	public static void main(String[] args) {
		if (args.length < ARGS_COUNT) {
			throw new IllegalArgumentException(
					String.format("Expected %d arguments, but got %d instead", ARGS_COUNT, args.length));
		}
		rng.setSeed(Arrays.hashCode(args));
		int maxN = Integer.parseInt(args[0]);
		int maxEdges = Integer.parseInt(args[1]);
		int maxColors = Integer.parseInt(args[2]);

		List<Graph> graphs = new ArrayList<>();
		int sumM = 0;
		int sumN = 0;
		while (sumN + 2 <= MAX && sumM + 2 <= MAX) {
			int n = 2 + rng.nextInt(maxN);
			int m = 2 + rng.nextInt(maxEdges);
			n = Math.min(n, MAX - sumN);
			m = Math.min(m, MAX - sumM);
			int c = 1 + rng.nextInt(Math.min(maxColors, m));

			Graph graph = new Graph(n);
			for (int i = 0; i < m; i++) {
				int from = rng.nextInt(n);
				int to = rng.nextInt(n);
				while (from == to) {
					from = rng.nextInt(n);
					to = rng.nextInt(n);
				}
				graph.addEdge(from, to, rng.nextInt(c));
			}
			graphs.add(graph);
			sumN += n;
			sumM += m;
		}
		printGraphs(graphs);
	}

	static void printGraphs(List<Graph> graphs) {
		try (PrintWriter out = new PrintWriter(System.out)) {
			out.println(graphs.size());
			graphs.forEach(g -> g.print(out));
		}
	}

	static class Graph {
		class Edge {
			int from, to, color;

			public Edge(int from, int to, int color) {
				this.from = from;
				this.to = to;
				this.color = color;
			}
		}

		int n;
		List<Edge> edges;
		public Graph(int n) {
			this.n = n;
			this.edges = new ArrayList<>();
		}

		void addEdge(int from, int to, int color) {
			edges.add(new Edge(from, to, color));
		}

		public void print(PrintWriter out) {
			out.println(n + " " + edges.size());
			edges.forEach(e -> out.println(String.format("%d %d %d", e.from + 1, e.to + 1, e.color + 1)));
		}
	}
}
