import java.io.PrintWriter;
import java.util.*;

public class GenSpanningTree {

    public final static Random rng = new Random("alternatingcycle".hashCode());
    public final static int MAX = 200_000;

    public final static int ARGS_COUNT = 3;

    public static void main(String[] args) {
        if (args.length < ARGS_COUNT) {
            throw new IllegalArgumentException(
                    String.format("Expected %d arguments, but got %d instead", ARGS_COUNT, args.length));
        }
        rng.setSeed(Arrays.hashCode(args));
        int n = Integer.parseInt(args[0]);
        int m = Integer.parseInt(args[1]);
        int c = Integer.parseInt(args[2]);
        
        List<Graph> graphs = new ArrayList<>();
        int sumM = 0;
        int sumN = 0;
        while (sumN + n <= MAX && sumM + m <= MAX) {
            Graph graph = new Graph(n);
            DSU dsu = new DSU(n);
            for (int i = 0; i < n - 1; i++) {
                int from = rng.nextInt(n);
                int to = rng.nextInt(n);
                while (!dsu.unite(from, to)) {
                    from = rng.nextInt(n);
                    to = rng.nextInt(n);
                }
                graph.addEdge(from, to, 0);
                graph.addEdge(to, from, 0);
            }

            for (int i = 2 * (n - 1); i < m; i++) {
                int from = rng.nextInt(n);
                int to = rng.nextInt(n);
                while (from == to) {
                    from = rng.nextInt(n);
                    to = rng.nextInt(n);
                }
                graph.addEdge(from, to, rng.nextInt(c));
            }
            graph.shuffle(rng);
            graphs.add(graph);
            sumN += n;
            sumM += m;
        }
        printGraphs(graphs);
    }
    
    static class DSU {
        int[] p;
        public DSU(int n) {
            p = new int[n];
            for (int i = 0; i < p.length; i++) {
                p[i] = i;
            }
        }
        int get(int i) {
            if (p[i] == i) {
                return i;
            }
            return p[i] = get(p[i]);
        }
        boolean unite(int i, int j) {
            i = get(i);
            j = get(j);
            if (i != j) {
                p[i] = j;
                return true;
            }
            return false;
        }
    }

    static void printGraphs(List<Graph> graphs) {
        try (PrintWriter out = new PrintWriter(System.out)) {
            out.println(graphs.size());
            graphs.forEach(g -> g.print(out));
        }
    }

    static int[] randomPermutation(Random rng, int n) {
        int[] a = new int[n];
        for (int i = 1; i < n; i++) {
            int pos = rng.nextInt(i + 1);
            a[i] = a[pos];
            a[pos] = i;
        }
        return a;
    }

    static class Graph {
        class Edge {
            int from, to, color;

            public Edge(int from, int to, int color) {
                this.from = from;
                this.to = to;
                this.color = color;
            }
        }

        int n;
        List<Edge> edges;

        public Graph(int n) {
            this.n = n;
            this.edges = new ArrayList<>();
        }

        void addEdge(int from, int to, int color) {
            edges.add(new Edge(from, to, color));
        }

        void shuffle(Random rng) {
            int[] perm = randomPermutation(rng, n);
            int[] permC = randomPermutation(rng, edges.size());
            Collections.shuffle(edges, rng);
            edges.forEach(e -> {e.from = perm[e.from]; e.to = perm[e.to]; e.color = permC[e.color];});
        }

        public void print(PrintWriter out) {
            out.println(n + " " + edges.size());
            edges.forEach(e -> out.println(String.format("%d %d %d", e.from + 1, e.to + 1, e.color + 1)));
        }
    }
}
