import java.io.*;
import java.util.*;
import java.util.stream.Collectors;

public class cycle_av_tl_ve {

    class DirectedGraph {
        private static final int WHITE = 0;
        private static final int GREY = 1;
        private static final int BLACK = 2;

        List<Integer> from, to;

        public DirectedGraph() {
            from = new ArrayList<>();
            to = new ArrayList<>();
        }

        void addEdge(int from, int to) {
            this.from.add(from);
            this.to.add(to);
        }

        int[][] graph;
        int[] color;
        int[] stack;
        int stackSize;
        List<Integer> cycle;

        void dfs(int u) {
            color[u] = GREY;
            stack[stackSize++] = u;
            for (int t = 0; t < graph[u].length; t++) {
                int v = graph[u][t];
                if (color[v] == WHITE) {
                    dfs(v);
                    if (cycle != null) {
                        return;
                    }
                } else if (color[v] == GREY) {
                    cycle = new ArrayList<>();
                    for (int pos = stackSize - 1; pos >= 0; pos--) {
                        cycle.add(stack[pos]);
                        if (stack[pos] == v) {
                            break;
                        }
                    }
                    Collections.reverse(cycle);
                    return;
                }
            }
            stackSize--;
            color[u] = BLACK;
        }

        List<Integer> findCycle() {
            int n = from.isEmpty() ? 0 : Collections.max(from) + 1;
            n = Math.max(n, from.isEmpty() ? 0 : Collections.max(to) + 1);
            int[] deg = new int[n];
            for (int i = 0; i < from.size(); i++) {
                deg[from.get(i)]++;
            }
            graph = new int[n][];
            for (int i = 0; i < n; i++) {
                graph[i] = new int[deg[i]];
            }
            for (int i = from.size() - 1; i >= 0; i--) {
                int v = from.get(i);
                graph[v][--deg[v]] = to.get(i);
            }
            color = new int[n];
            stack = new int[n];
            stackSize = 0;
            for (int i = 0; i < n; i++) {
                if (color[i] == 0) {
                    dfs(i);
                    if (cycle != null) {
                        return cycle;
                    }
                }
            }
            return null;
        }

    }

    void solveOne() {
        int n = in.nextInt(), m = in.nextInt();
        List<Integer>[] outEdges = new List[n];
        for (int i = 0; i < n; i++) {
            outEdges[i] = new ArrayList<>();
        }
        int[] from = new int[m], to = new int[m], color = new int[m];
        for (int i = 0; i < m; i++) {
            from[i] = in.nextInt() - 1;
            to[i] = in.nextInt() - 1;
            color[i] = in.nextInt() - 1;
            outEdges[from[i]].add(i);
        }
        DirectedGraph graph = new DirectedGraph();
        for (int i = 0; i < m; i++) {
            for (int j : outEdges[to[i]]) {
                if (color[i] != color[j]) {
                    graph.addEdge(i, j);
                }
            }
        }

        List<Integer> cycle = graph.findCycle();
        if (cycle == null) {
            out.println(-1);
        } else {
            cycle = cycle.stream().filter(x -> x < m).collect(Collectors.toList());
            out.println(cycle.size());
            cycle.forEach(x -> out.print((x + 1) + " "));
            out.println();
        }

    }


    void solve() {
        int T = in.nextInt();
        while (T-- > 0) {
            solveOne();
        }
    }

    FastScanner in;
    PrintWriter out;

    void run() {
        in = new FastScanner();
        out = new PrintWriter(System.out);
        solve();
        out.close();
    }

    class FastScanner {
        BufferedReader br;
        StringTokenizer st;

        public FastScanner() {
            br = new BufferedReader(new InputStreamReader(System.in));
        }

        public FastScanner(String file) {
            try {
                br = new BufferedReader(new FileReader(file));
            } catch (FileNotFoundException e) {
                e.printStackTrace();
            }
        }

        public String nextToken() {
            while (st == null || !st.hasMoreElements()) {
                try {
                    st = new StringTokenizer(br.readLine());
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
            return st.nextToken();
        }

        public boolean hasMoreTokens() {
            while (st == null || !st.hasMoreElements()) {
                try {
                    String line = br.readLine();
                    if (line == null) {
                        return false;
                    }
                    st = new StringTokenizer(line);
                } catch (IOException e) {
                    return false;
                }
            }
            return true;
        }

        public int nextInt() {
            return Integer.parseInt(nextToken());
        }

        public long nextLong() {
            return Long.parseLong(nextToken());
        }

        public double nextDouble() {
            return Double.parseDouble(nextToken());
        }

        public int[] nextIntArray(int length) {
            int[] array = new int[length];
            for (int i = 0; i < length; i++) {
                array[i] = nextInt();
            }
            return array;
        }
    }

    public static void main(String[] args) {
        new cycle_av_tl_ve().run();
    }
}
