#include "testlib.h"
#include <iostream>

using namespace std;

int dsu_get(vector<int> &p, int x) {
	return p[x] == x ? x : (p[x] = dsu_get(p, p[x]));
}

void dsu_unite(vector<int> &p, int x, int y) {
	p[dsu_get(p, x)] = dsu_get(p, y);
}

int get_id(int x, int y, int m) {
	return x * m + y;
}

bool check_connected(vector<string> res, char c) {
	int n = res.size();
	int m = res[0].length();
	vector<int> p(n * m);
	for (int i = 0; i < p.size(); i++) {
		p[i] = i;
	}
	for (int i = 0; i < n; i++) {
		for (int j = 0; j < m; j++) {
			for (int dx = 0; dx < 2; dx++) {
				int dy = 1 - dx;
				int ni = (i + dx) % n;
				int nj = (j + dy) % m;
				if (res[i][j] == c && res[ni][nj] == c) {
					dsu_unite(p, get_id(i, j, m), get_id(ni, nj, m));
				}
			}
		}
	}
	int need_p = -1;
	for (int i = 0; i < n; i++) {
		for (int j = 0; j < m; j++) {
			if (res[i][j] == c) {
				int my_id = dsu_get(p, get_id(i, j, m));
				if (need_p == -1) {
					need_p = my_id;
				}
				if (need_p != my_id) {
					return false;
				}
			}
		}
	}
	return true;
}

int main(int argc, char * argv[])
{
    registerTestlibCmd(argc, argv);

    int n = inf.readInt();
    int m = inf.readInt();

    vector<vector<int>> a(n);
    for (int i = 0; i < n; i++) {
    	a[i].reserve(m);
    	for (int j = 0; j < m; j++) {
    		a[i][j] = inf.readInt();
    	}
    }

    vector<string> res(4 * n);
    for (int i = 0; i < 4 * n; i++) {
    	res[i] = ouf.readWord();
    	if (res[i].length() != 4 * m) {
    		quitf(_wa, "Lenght of %d-th line is not %d (it is %d)", i + 1, 4 * m, (int) res[i].length());
    	}
    	for (int j = 0; j < res[i].length(); j++) {
    		if (res[i][j] != '.' && res[i][j] != '*') {
    			quitf(_wa, "res[%d][%d] is not a valid answer character (%c)", i + 1, j + 1, res[i][j]);
    		}
    	}
    }

    for (int i = 0; i < n; i++) {
    	for (int j = 0; j < m; j++) {
    		int sum = 0;
    		for (int dx = 0; dx < 4; dx++) {
    			for (int dy = 0; dy < 4; dy++) {
    				if (res[i * 4 + dx][j * 4 + dy] == '*') {
    					sum++;
    				}
    			}
    		}
    		if (sum != a[i][j]) {
    			quitf(_wa, "Sum of '#' in (%d, %d) is not correct: expected %d, found %d", i + 1, j + 1, a[i][j], sum);
    		}
    	}
    }

    if (!check_connected(res, '.')) {
    	quitf(_wa, "'.' symbols is not connected");
    }

    if (!check_connected(res, '*')) {
    	quitf(_wa, "'*' symbols is not connected");
    }

    quitf(_ok, "Ok");
}
