import ru.ifmo.testlib.*;
import static ru.ifmo.testlib.Outcome.Type.*;

public class Check implements Checker {
	public static final int MAX_NEW = 280;
	
	@Override
	public Outcome test(InStream inf, InStream ouf, InStream ans) {
		Chain input;
		try {
			input = readChain(inf);
		} catch (Exception e) {
			return new Outcome(FAIL, "Invalid input", e);
		}
		int ansSize = ans.nextInt();
		Chain chain = readChain(ouf);
		if (!chain.text.equals(input.text)) {
			return new Outcome(WA, "Not the same text");
		}
		if (chain.parts < ansSize) {
			return new Outcome(FAIL, "Better answer: " + chain.parts);
		}
		if (chain.parts > ansSize) {
			return new Outcome(WA, "Non-optimal answer: " + chain.parts);
		}
		return new Outcome(OK, "Correct answer: " + chain.parts);
	}
	
	public Chain readChain(InStream stream) {
		int n = stream.nextInt();
		stream.nextLine();
		StringBuilder sb = new StringBuilder();
		for (int i = 0; i < n; i++) {
			String s = stream.nextLine();
			if (s.length() > MAX_NEW) {
				throw new Outcome(WA, "Line is too long: " + s.length());
			}
			String[] tokens = s.split(" ");
			for (int j = 0; j < tokens.length - 1; j++) {
				if (tokens[j].length() == 0) {
					throw new Outcome(WA, "Invalid spacing in " + s);
				}
				sb.append(tokens[j]).append(" ");
			}
			if (!tokens[tokens.length - 1].equals("(" + (i + 1) + "/" + n + ")")) {
				throw new Outcome(WA, "Invalid ending: " + tokens[tokens.length - 1]);
			}
		}
		return new Chain(n, sb.toString());
	}
	
	class Chain {
		int parts;
		String text;
		
		public Chain(int parts, String text) {
			this.parts = parts;
			this.text = text;
		}
	}
}
