import java.io.*;
import java.util.*;

public class Tests {
	private static File dir = new File(".");
	private int curTest = 1;
	private static boolean verbose = false;
	
	public static final int MAX_N = 5000;
	public static final int MAX_TEN = 1000; // max power of ten <= MAX_N
	public static final int MAX_OLD = 140;
	public static final int MAX_NEW = 280;
	public static final int MAX_TEXT = MAX_OLD - 4 - 2 * ("" + MAX_N).length();

	private String testName;
	private PrintWriter out;
	private Random r = new Random(587043274598334L);
	
	boolean isValidChar(char c) {
		return 32 <= c && c < 64 || 'a' <= c && c <= 'z' || 'A' <= c && c <= 'Z' || '0' <= c && c <= '9';
	}

	public void run() {
		t("Winter!.. The peasant breathes a sigh,", "renews his sledge, and makes his way.");
		t("An example with sylla-", "bification and punctuation", "!");
		t("9");
		t("(");
		t("(", "7", "/", "7", ")");
		t("(2/3)");
		t("(9/8)", "(9/8)");
		t("(4/4)", "((3//4))", ")2/4(", "( 1 / 4 )");
		t("t(\"(4/4)\", \"(3/4)\", \"(2/4)\", \"( 1 / 4 )\");");
		tExact(566, true);
		tRandom(MAX_N, 1, 2);
		tRandom(MAX_N, 1, 3);
		tRandom(MAX_N, 1, MAX_TEXT + 5);
		tRandom(MAX_N, MAX_TEXT, MAX_TEXT + 5);
		tRandom(MAX_N, MAX_TEXT, MAX_TEXT + 1);
		tRandom(MAX_N, MAX_TEXT / 2, MAX_TEXT / 2 + 1);
		tRandom(MAX_N, MAX_TEXT / 2 + 1, MAX_TEXT / 2 + 2);
		tMax(MAX_N);
		tMax(MAX_TEN);
		tOneWord(MAX_TEN, 1);
		tOneWord(MAX_TEN, MAX_OLD / 3);
		tOneWord(MAX_N, 1);
		tOneWord(MAX_N, 2);
		tExact(9, true);
		tExact(10, false);
		tExact(99, true);
		tExact(100, false);
		tExact(MAX_TEN - 1, true);
		tExact(MAX_TEN, false);
	}

	private void tRandom(int n, int lenFrom, int lenTo) {
		String[] s = new String[n];
		for (int i = 0; i < s.length; i++) {
			StringBuilder sb = new StringBuilder();
			int maxLen = MAX_OLD - 4 - ("" + (i + 1)).length() - ("" + s.length).length();
			for (;;) {
				int len = r.nextInt(lenTo - lenFrom) + lenFrom;
				if (sb.length() + len > maxLen) {
					if (sb.length() > 0) {
						break;
					} else {
						continue;
					}
				}
				String next = randomWord(len);
				sb.append(next).append(" ");
			}
			s[i] = sb.toString().trim();
		}
		t(s);
	}
	
	private void tMax(int n) {
		String[] s = new String[n];
		for (int i = 0; i < s.length; i++) {
			StringBuilder sb = new StringBuilder();
			int maxLen = MAX_OLD - 4 - ("" + (i + 1)).length() - ("" + s.length).length();
			while (sb.length() + 1 <= maxLen) {
				sb.append("(");
			}
			s[i] = sb.toString();
		}
		t(s);
	}
	
	private void tOneWord(int n, int len) {
		String[] s = new String[n];
		for (int i = 0; i < s.length; i++) {
			s[i] = randomWord(len);
		}
		t(s);
	}
	
	private void tExact(int n, boolean mode) {
		int tokens = 0;
		for (int i = 0; i < n; i++) {
			if (i == n - 1 && !mode) {
				tokens += 1;
				continue;
			}
			int maxLen = MAX_NEW - 4 - ("" + (i + 1)).length() - ("" + n).length();
			tokens += (maxLen + 1) / 2;
		}
		int each = tokens <= MAX_N ? 1 : MAX_OLD / 3;
		ArrayList<String> s = new ArrayList<>();
		while (tokens > 0) {
			int t = Math.min(tokens, each);
			StringBuilder sb = new StringBuilder();
			for (int i = 0; i < t; i++) {
				sb.append(randomChar()).append(' ');
			}
			tokens -= t;
			s.add(sb.toString().trim());
		}
		t(s.toArray(new String[s.size()]));
	}
	
	public String randomWord(int len) {
		StringBuilder sb = new StringBuilder();
		while (len-- > 0) {
			sb.append(randomChar());
		}
		return sb.toString();
	}
	
	public char randomChar() {
		for (;;) {
			char c = (char) r.nextInt(128);
			if (c != ' ' && isValidChar(c)) {
				return c;
			}
		}
	}

	private void t(String... lines) {
		open();
		out.println(lines.length);
		for (int i = 0; i < lines.length; i++) {
			out.println(lines[i] + " (" + (i + 1) + "/" + lines.length + ")");
		}
		close();
	}

	private void open() {
		try {
			testName = "" + curTest / 10 + curTest % 10;
			out = new PrintWriter(new File(dir, testName));
			if (verbose) { 
				System.out.print("Generating " + testName + "...");
			}
		} catch (IOException e) {
			throw new RuntimeException("Unable to open " + testName + " for writing", e);
		}
	}

	private void close() {
		out.close();
		if (verbose) {
			System.out.println(" done.");
		}
		curTest++;
	}

	public static void main(String[] args) {
		new Tests().run();
	}
}
